@@ -11,14 +11,18 @@ import traceback
 import stem.util.term
 
 import darc.typing as typing
-from darc.const import DEBUG, PATH_ID, PATH_LN
-from darc.db import redis_command, save_requests
+from darc.const import DB, DB_WEB, DEBUG, FLAG_DB, PATH_ID, PATH_LN
+from darc.db import _redis_command, save_requests
 from darc.link import parse_link
+from darc.model import (HostnameModel, HostnameQueueModel, HostsModel, RequestsHistoryModel,
+                        RequestsModel, RequestsQueueModel, RobotsModel, SeleniumModel,
+                        SeleniumQueueModel, SitemapModel, URLModel)
 from darc.process import process
 from darc.proxy.freenet import _FREENET_PROC
 from darc.proxy.i2p import _I2P_PROC
 from darc.proxy.tor import _TOR_CTRL, _TOR_PROC
 from darc.proxy.zeronet import _ZERONET_PROC
+from darc.submit import SAVE_DB
 
 # wait for Redis connection?
 _WAIT_REDIS = bool(int(os.getenv('DARC_REDIS', '1')))
@@ -79,15 +83,37 @@ def main():
 
     # wait for Redis
     if _WAIT_REDIS:
-        redis_command('set', 'darc', pid)
+        if not FLAG_DB:
+            _redis_command('set', 'darc', pid)
+
+    if FLAG_DB:
+        while True:
+            with contextlib.suppress(Exception):
+                with DB:
+                    DB.create_tables([
+                        HostnameQueueModel, RequestsQueueModel, SeleniumQueueModel,
+                    ])
+            break
+
+    if SAVE_DB:
+        while True:
+            with contextlib.suppress(Exception):
+                with DB_WEB:
+                    DB_WEB.create_tables([
+                        HostnameModel, URLModel,
+                        RobotsModel, SitemapModel, HostsModel,
+                        RequestsModel, RequestsHistoryModel, SeleniumModel,
+                    ])
+            break
 
     if DEBUG:
         print(stem.util.term.format('-*- Initialisation -*-', stem.util.term.Color.MAGENTA))  # pylint: disable=no-member
 
         # nuke the db
-        redis_command('delete', 'queue_hostname')
-        redis_command('delete', 'queue_requests')
-        redis_command('delete', 'queue_selenium')
+        if not FLAG_DB:
+            _redis_command('delete', 'queue_hostname')
+            _redis_command('delete', 'queue_requests')
+            _redis_command('delete', 'queue_selenium')
 
     link_list = list()
     for link in filter(None, map(lambda s: s.strip(), args.link)):
