@@ -3,10 +3,11 @@
 =======================
 
 The :func:`~darc.error.render_error` function can be used to render
-multi-line error messages with |term|_ colours.
+multi-line error messages with :mod:`stem.util.term` colours.
 
 The :mod:`darc` project provides following custom exceptions:
 
+* :exc:`~darc.error.LinkNoReturn`
 * :exc:`~darc.error.UnsupportedLink`
 * :exc:`~darc.error.UnsupportedPlatform`
 * :exc:`~darc.error.UnsupportedProxy`
@@ -19,6 +20,9 @@ The :mod:`darc` project provides following custom exceptions:
 * :exc:`~darc.error.FreenetBootstrapFailed`
 * :exc:`~darc.error.APIRequestFailed`
 * :exc:`~darc.error.SiteNotFoundWarning`
+* :exc:`~darc.error.LockWarning`
+* :exc:`~darc.error.TorRenewFailed`
+* :exc:`~darc.error.RedisCommandFailed`
 
 """
 
@@ -27,6 +31,10 @@ import stem.util.term
 import darc.typing as typing
 
 
+class LinkNoReturn(Exception):
+    """The link has no return value from the hooks."""
+
+
 class UnsupportedLink(Exception):
     """The link is not supported."""
 
@@ -43,6 +51,10 @@ class TorBootstrapFailed(Warning):
     """Tor bootstrap process failed."""
 
 
+class TorRenewFailed(Warning):
+    """Tor renew request failed."""
+
+
 class I2PBootstrapFailed(Warning):
     """I2P bootstrap process failed."""
 
@@ -55,6 +67,10 @@ class FreenetBootstrapFailed(Warning):
     """Freenet bootstrap process failed."""
 
 
+class RedisCommandFailed(Warning):
+    """Redis command execution failed."""
+
+
 class APIRequestFailed(Warning):
     """API submit failed."""
 
@@ -63,16 +79,20 @@ class SiteNotFoundWarning(ImportWarning):
     """Site customisation not found."""
 
 
+class LockWarning(Warning):
+    """Failed to acquire Redis lock."""
+
+
 def render_error(message: str, colour: typing.Color) -> str:
     """Render error message.
 
-    The function wraps the |format|_ function to provide multi-line
-    formatting support.
+    The function wraps the :func:`stem.util.term.format` function to
+    provide multi-line formatting support.
 
     Args:
         message: Multi-line message to be rendered with ``colour``.
         colour (stem.util.term.Color): Front colour of text, c.f.
-            |color|_.
+            :class:`stem.util.term.Color`.
 
     Returns:
         The rendered error message.
