@@ -18,11 +18,11 @@
 # -- Project information -----------------------------------------------------
 
 project = 'darc'
-copyright = '2020, Jarry Shaw'
+copyright = '2019-2020, Jarry Shaw'
 author = 'Jarry Shaw'
 
 # The full version, including alpha/beta/rc tags
-release = '0.5.0.post1'
+release = '0.6.0.post1'
 
 
 # -- General configuration ---------------------------------------------------
@@ -102,6 +102,10 @@ html_static_path = ['_static']
 
 # -- Customised hooks --------------------------------------------------------
 
+import os
+
+os.environ['TOR_PASS'] = 'null'
+
 
 def maybe_skip_member(app, what: str, name: str, obj: object, skip: bool, options: dict):
     if name == '_abc_impl':
