@@ -19,9 +19,16 @@ SCPT = os.path.join(ROOT, 'upload.sh')
 
 def check_call(*args, **kwargs):
     """Wraps :func:`subprocess.check_call`."""
-    while True:
-        with contextlib.suppress(subprocess.CalledProcessError):
-            return subprocess.check_call(*args, **kwargs)
+    with open('logs/upload.log', 'at', buffering=1) as file:
+        if 'stdout' not in kwargs:
+            kwargs['stdout'] = file
+        if 'stderr' not in kwargs:
+            kwargs['stderr'] = subprocess.STDOUT
+
+        for _ in range(3):
+            with contextlib.suppress(subprocess.CalledProcessError):
+                return subprocess.check_call(*args, **kwargs)
+            time.sleep(60)
 
 
 def upload(file, path, host, user):
@@ -32,11 +39,12 @@ def upload(file, path, host, user):
     print(f'[{datetime.datetime.now().isoformat()}] Archiving & uploading APi submission files...')
 
     check_call(['docker-compose', '--file', file, 'pause'])
-    with contextlib.suppress(subprocess.CalledProcessError):
-        subprocess.check_call(['bash', SCPT], env=os.environ.update(dict(
-            HOST=host,
-            USER=user,
-        )), cwd=path)
+    with open('logs/upload.log', 'at', buffering=1) as log_file:
+        with contextlib.suppress(subprocess.CalledProcessError):
+            subprocess.check_call(['bash', SCPT], env=os.environ.update(dict(
+                HOST=host,
+                USER=user,
+            )), cwd=path, stdout=log_file, stderr=subprocess.STDOUT)
     os.makedirs(path_api, exist_ok=True)
     check_call(['docker-compose', '--file', file, 'unpause'])
 
@@ -73,16 +81,27 @@ def main():
     if args.interval <= 0:
         parser.error('invalid interval')
 
-    # sleep before first round
-    time.sleep(args.interval)
+    os.makedirs('logs', exist_ok=True)
+    if os.path.isfile('logs/upload.log'):
+        os.rename('logs/upload.log', f'logs/upload-{time.strftime(r"%Y%m%d-%H%M%S")}.log')
 
-    while True:
-        try:
-            with contextlib.suppress(Exception):
-                upload(args.file, args.path, args.host, args.user)
-        except KeyboardInterrupt:
-            break
-        time.sleep(args.interval)
+    with open('logs/upload.log', 'at', buffering=1) as file:
+        date = datetime.datetime.now().ctime()
+        print('-' * len(date), file=file)
+        print(date, file=file)
+        print('-' * len(date), file=file)
+
+        with contextlib.redirect_stdout(file), contextlib.redirect_stderr(file):
+            ## sleep before first round
+            #time.sleep(args.interval)
+
+            while True:
+                try:
+                    with contextlib.suppress(Exception):
+                        upload(args.file, args.path, args.host, args.user)
+                except KeyboardInterrupt:
+                    break
+                time.sleep(args.interval)
     return 0
 
 
